@@ -52,18 +52,9 @@
 #define TIMER_STATUS_TIMEOUT_MS 5000 // 5s will message in status bar shown
 #define TIMER_MARK_MSG_READ_MS 5000 /* Mark message as read after 5 seconds. */
 
-#if !defined(Q_OS_WIN)
-    /* "file://localhost" */
-#  define URL_FILE_PREFIX "file://"
-#else /* defined(Q_OS_WIN) */
-    /* "file://localhost/" */
-#  define URL_FILE_PREFIX "file:///"
-#endif /* !defined(Q_OS_WIN) */
-
 #define CZ_NIC_URL "https://www.nic.cz"
 #define DATOVKA_ONLINE_HELP_URL "https://gitlab.labs.nic.cz/labs/qdatovka/wikis/manual"
 #define DATOVKA_HOMEPAGE_URL "https://labs.nic.cz/cs/datovka.html"
-#define DATOVKA_OFFLINE_HELP_URL URL_FILE_PREFIX "help/index.html"
 #define DATOVKA_CHECK_NEW_VERSION_URL "https://secure.nic.cz/files/datove_schranky/Version"
 #define DATOVKA_DOWNLOAD_URL "https://labs.nic.cz/cs/datovka.html"
 #define PWD_EXPIRATION_NOTIFICATION_DAYS 7 // show expiration date dialog before xx days
@@ -117,18 +108,11 @@ enum MessageDirection {
  * Defined roles across the application.
  */
 #define ROLE_PLAIN_DISPLAY (Qt::UserRole + 1) /* Low level data. */
-#define ROLE_ACNT_USER_NAME (Qt::UserRole + 2) /*
-                                                * Used to access the user name.
-                                                */
-#define ROLE_ACNT_UNREAD_MSGS (Qt::UserRole + 3) /*
-                                                  * Used to store number of
-                                                  * unread messages.
-                                                  */
-#define ROLE_MSGS_DB_ENTRY_TYPE (Qt::UserRole + 4) /*
+#define ROLE_MSGS_DB_ENTRY_TYPE (Qt::UserRole + 2) /*
                                                     * Used to determine the db
                                                     * data type of the column.
                                                     */
-#define ROLE_MSGS_DB_PROXYSORT (Qt::UserRole + 5) /*
+#define ROLE_MSGS_DB_PROXYSORT (Qt::UserRole + 3) /*
                                                    * Used for sorting according
                                                    * to boolean values which
                                                    * are displayed as icons.
@@ -240,6 +224,11 @@ QString convertEventTypeToString(int value);
  */
 QString convertUserPrivilsToString(int userPrivils);
 
+/*!
+ * @brief Return sender databox type as string from number representation.
+ */
+QString convertSenderDbTypesToString(int value);
+
 /*!
  * @brief Convert state box to text
  */
