@@ -129,15 +129,17 @@ private slots:
 	void processSingleInstanceMessages(const QString &message);
 
 	/*!
-	 * @brief Workers finished.
+	 * @brief Fired when workers performing tasks on background have
+	 *     finished.
 	 */
-	void workersFinished(void);
+	void backgroundWorkersFinished(void);
 
 	/*!
 	 * @brief Performs action depending on message download outcome.
 	 */
 	void collectDownloadMessageStatus(const QString &usrName, qint64 msgId,
-	    const QDateTime &deliveryTime, int result, const QString &errDesc);
+	    const QDateTime &deliveryTime, int result, const QString &errDesc,
+	    bool listScheduled);
 
 	/*!
 	 * @brief Performs action depending on message list download outcome.
@@ -159,16 +161,11 @@ private slots:
 	 */
 	void datovkaVersionResponce(QNetworkReply* reply);
 
-	/*!
-	 * @brief Updates the account model according to the change properties.
-	 */
-	void updateAccountListEntry(const QString &userName);
-
 	/*!
 	 * @brief Verify whether a connection to ISDS can be establisehd
 	 *     and databox exists for the account.
 	 */
-	void getAccountUserDataboxInfo(AccountModel::SettingsMap accountInfo);
+	void getAccountUserDataboxInfo(AcntSettings accountInfo);
 
 	/*!
 	 * @brief Redraws widgets according to selected account item.
@@ -199,6 +196,11 @@ private slots:
 	 */
 	void messageItemRightClicked(const QPoint &point);
 
+	/*!
+	 * @brief Handle message double click.
+	 */
+	void viewSelectedMessage(void);
+
 	/*!
 	 * @brief Saves message selection.
 	 */
@@ -238,11 +240,6 @@ private slots:
 	 */
 	void attachmentItemRightClicked(const QPoint &point);
 
-	/*!
-	 * @brief Handle attachment double click.
-	 */
-	void attachmentItemDoubleClicked(const QModelIndex &index);
-
 	/*!
 	 * @brief Saves selected attachments to file.
 	 */
@@ -380,8 +377,13 @@ private slots:
 
 	/*!
 	 * @brief Downloads new messages from server for selected account.
+	 *
+	 * @param[in] userName Account user name. If empty string is supplied,
+	 *                     then selected account will be determined from
+	 *                     the account list.
+	 * @return True if some actions have been planned.
 	 */
-	void synchroniseSelectedAccount(void);
+	bool synchroniseSelectedAccount(QString userName = QString());
 
 	/*!
 	 * @brief Downloads the attachments for the selected message.
@@ -493,6 +495,11 @@ private slots:
 	 */
 	void exportSelectedMessageEnvelopesAsPDF(void);
 
+	/*!
+	 * @brief Export selected message envelope as PDF and attachment files.
+	 */
+	void exportSelectedMessageEnvelopeAttachments(void);
+
 	/*!
 	 * @brief Sends selected messages as ZFO into default e-mail client.
 	 */
@@ -684,6 +691,16 @@ private slots:
 	void doActionAfterSentMsgSlot(const QString &userName,
 	    const QString &lastDir);
 
+	/*!
+	 * @brief Show tags manager dialog for tags settings.
+	 */
+	void showTagDlg(void);
+
+	/*!
+	 * @brief Add/detele tags to/from selected messages.
+	 */
+	void addOrDeleteMsgTags(void);
+
 private:
 
 	QTimer m_timerSyncAccounts;
@@ -693,6 +710,14 @@ private:
 
 	void showStatusTextPermanently(const QString &qStr);
 
+	/*!
+	 * @brief Used to view selected message via event filter.
+	 *
+	 * @param[in] mwPtr Pointer to main window.
+	 */
+	static
+	void viewSelectedMessageViaFilter(QObject *mwPtr);
+
 	/*!
 	 * @brief Export message into ZFO file dialogue.
 	 */
@@ -723,6 +748,13 @@ private:
 	    const QString &userName, qint64 dmId, QDateTime deliveryTime,
 	    bool askLocation);
 
+	/*!
+	 * @brief Export selected message envelope as PDF and attachment files.
+	 */
+	void exportMessageEnvelopeAttachments(const QString &attachPath,
+	    const QString &userName, qint64 dmId, QDateTime deliveryTime,
+	    bool askLocation);
+
 	/*!
 	 * @brief Set info status bar from worker.
 	 */
@@ -917,7 +949,7 @@ private:
 	 *
 	 * @note Does not add/remove any nodes, just updates the counts.
 	 */
-	bool updateExistingAccountModelUnread(QModelIndex index);
+	bool updateExistingAccountModelUnread(const QModelIndex &index);
 
 	/*!
 	 * @brief Partially regenerates account model according to the database
@@ -929,7 +961,7 @@ private:
 	 * @note This function adds/removes nodes and does not set the
 	 *     currentIndex back to its original position.
 	 */
-	bool regenerateAccountModelYears(QModelIndex index);
+	bool regenerateAccountModelYears(const QModelIndex &index);
 
 	/*!
 	 * @brief Regenerates account model according to the database content.
@@ -966,22 +998,6 @@ private:
 	 */
 	QString createDatovkaBanner(const QString &version) const;
 
-	/*!
-	 * @brief Returns user name related to given account item.
-	 *
-	 * @note If accountItem is 0 then currently selected account is
-	 *     processed.
-	 */
-	QString userNameFromItem(const QStandardItem *accountItem = 0) const;
-
-	/*!
-	 * @brief Returns pointer to account item related to given user name.
-	 *
-	 * @param[in] userName User login name.
-	 * @return 0 when no such top item found.
-	 */
-	QStandardItem *itemFromUserName(const QString &userName) const;
-
 	/*!
 	 * @brief Set export paths to selected account item.
 	 */
@@ -990,9 +1006,11 @@ private:
 	/*!
 	 * @brief Delete message from long term storage in ISDS and
 	 * local database - based on delFromIsds parameter.
+	 *
+	 * @return True if message has been removed from local database.
 	 */
-	qdatovka_error eraseMessage(const QString &userName, qint64 dmId,
-	    const QDateTime &deliveryTime, bool delFromIsds);
+	bool eraseMessage(const QString &userName, qint64 dmId,
+	     const QDateTime &deliveryTime, bool delFromIsds);
 
 	/*!
 	 * @brief Authenticate message from ZFO file.
@@ -1015,23 +1033,20 @@ private:
 	/*!
 	 * @brief connect to ISDS databox from new account
 	 */
-	bool firstConnectToIsds(AccountModel::SettingsMap &accountInfo,
-	    bool showDialog);
+	bool firstConnectToIsds(AcntSettings &accountInfo, bool showDialog);
 
 	/*!
 	 * @brief Login to ISDS server by user name and password only.
 	 */
 	static
-	bool loginMethodUserNamePwd(
-	    AccountModel::SettingsMap &accountInfo,
+	bool loginMethodUserNamePwd(AcntSettings &accountInfo,
 	    MainWindow *mw, const QString &pwd = QString());
 
 	/*!
 	 * @brief Login to ISDS server by user name, password and OTP code.
 	 */
 	static
-	bool loginMethodUserNamePwdOtp(
-	    AccountModel::SettingsMap &accountInfo,
+	bool loginMethodUserNamePwdOtp(AcntSettings &accountInfo,
 	    MainWindow *mw, const QString &pwd = QString(),
 	    const QString &otp = QString());
 
@@ -1057,15 +1072,14 @@ private:
 	 * @brief Login to ISDS server by certificate only.
 	 */
 	static
-	bool loginMethodCertificateOnly(AccountModel::SettingsMap &accountInfo,
+	bool loginMethodCertificateOnly(AcntSettings &accountInfo,
 	    MainWindow *mw, const QString &key = QString());
 
 	/*!
 	 * @brief Login to ISDS server by certificate, user name and password.
 	 */
 	static
-	bool loginMethodCertificateUserPwd(
-	    AccountModel::SettingsMap &accountInfo,
+	bool loginMethodCertificateUserPwd(AcntSettings &accountInfo,
 	    MainWindow *mw, const QString &pwd = QString(),
 	    const QString &key = QString());
 
@@ -1073,8 +1087,8 @@ private:
 	 * @brief Login to ISDS server by certificate and data box ID.
 	 */
 	static
-	bool loginMethodCertificateIdBox(
-	    AccountModel::SettingsMap &accountInfo, MainWindow *mw);
+	bool loginMethodCertificateIdBox(AcntSettings &accountInfo,
+	    MainWindow *mw);
 
 	/*!
 	 * @brief Sent and check a new version of Datovka.
@@ -1223,6 +1237,8 @@ private:
 	QString m_import_zfo_path;
 	bool isMainWindow;
 
+	QStringList m_msgTblAppendedCols; /*< Appended columns. */
+
 	/* User interface elements. */
 	Ui::MainWindow *ui; /*!< User interface as generated from ui files. */
 	QLineEdit *mui_filterLine; /*!< Search filter line object. */
@@ -1246,6 +1262,7 @@ private:
 	 * @brief Sets action icons.
 	 */
 	void setMenuActionIcons(void);
+
 };
 
 
