@@ -27,10 +27,9 @@
 #include "dlg_create_account.h"
 #include "src/io/dbs.h"
 #include "src/log/log.h"
-#include "src/models/accounts_model.h"
 
 
-DlgCreateAccount::DlgCreateAccount(const AccountModel::SettingsMap &accountInfo,
+DlgCreateAccount::DlgCreateAccount(const AcntSettings &accountInfo,
     Action action, QWidget *parent)
     : QDialog(parent),
     m_accountInfo(accountInfo),
@@ -100,6 +99,7 @@ void DlgCreateAccount::setCurrentAccountData(void)
 		this->accountLineEdit->setText(m_accountInfo.accountName());
 		this->usernameLineEdit->setText(m_accountInfo.userName());
 		this->usernameLineEdit->setEnabled(false);
+		this->testAccountCheckBox->setEnabled(false);
 		break;
 	case ACT_PWD:
 		this->setWindowTitle(tr("Enter password for account") + " "
@@ -109,8 +109,8 @@ void DlgCreateAccount::setCurrentAccountData(void)
 		this->infoLabel->setEnabled(false);
 		this->loginmethodComboBox->setEnabled(false);
 		this->usernameLineEdit->setText(m_accountInfo.userName());
-		this->testAccountCheckBox->setEnabled(false);
 		this->usernameLineEdit->setEnabled(false);
+		this->testAccountCheckBox->setEnabled(false);
 		this->addCertificateButton->setEnabled(false);
 		break;
 	case ACT_CERT:
@@ -121,8 +121,8 @@ void DlgCreateAccount::setCurrentAccountData(void)
 		this->infoLabel->setEnabled(false);
 		this->loginmethodComboBox->setEnabled(false);
 		this->usernameLineEdit->setText(m_accountInfo.userName());
-		this->testAccountCheckBox->setEnabled(false);
 		this->usernameLineEdit->setEnabled(false);
+		this->testAccountCheckBox->setEnabled(false);
 		this->passwordLineEdit->setEnabled(false);
 		break;
 	case ACT_CERTPWD:
@@ -133,8 +133,8 @@ void DlgCreateAccount::setCurrentAccountData(void)
 		this->infoLabel->setEnabled(false);
 		this->loginmethodComboBox->setEnabled(false);
 		this->usernameLineEdit->setText(m_accountInfo.userName());
-		this->testAccountCheckBox->setEnabled(false);
 		this->usernameLineEdit->setEnabled(false);
+		this->testAccountCheckBox->setEnabled(false);
 		break;
 	case ACT_IDBOX:
 		this->setWindowTitle(tr("Enter ID of your databox for account")
@@ -261,6 +261,7 @@ void DlgCreateAccount::setActiveButton(int itemindex)
 		this->rememberPswcheckBox->setEnabled(true);
 	}
 	this->usernameLineEdit->setEnabled(m_action == ACT_ADDNEW);
+	this->testAccountCheckBox->setEnabled(m_action == ACT_ADDNEW);
 
 	m_loginmethod = itemindex;
 	checkInputFields();
@@ -276,7 +277,7 @@ void DlgCreateAccount::saveAccount(void)
 {
 	debugSlotCall();
 
-	AccountModel::SettingsMap itemSettings;
+	AcntSettings itemSettings;
 	const QString userName(m_accountInfo.userName());
 
 	/* set account index, itemTop and map itemSettings for account */
@@ -338,13 +339,14 @@ void DlgCreateAccount::saveAccount(void)
 	case ACT_CERT:
 	case ACT_CERTPWD:
 	case ACT_IDBOX:
-		/* TODO -- Update account model according to settings. */
 		AccountModel::globAccounts[userName] = itemSettings;
 		/*
 		 * Catching the following signal is required only when
 		 * ACT_EDIT was enabled.
+		 *
+		 * The account model catches the signal.
 		 */
-		emit changedAccountProperties(userName);
+		emit AccountModel::globAccounts.accountDataChanged(userName);
 		/* TODO -- Save/update related account DB entry? */
 		break;
 	case ACT_ADDNEW:
