@@ -30,7 +30,6 @@
 #include <QTimer>
 
 #include "src/common.h"
-#include "src/models/accounts_model.h"
 #include "ui_dlg_ds_search.h"
 
 
@@ -54,12 +53,14 @@ private slots:
 	void searchDataBox(void);
 	void pingIsdsServer(void);
 	void setFirtsColumnActive(void);
+	void contactItemDoubleClicked(const QModelIndex &index);
 
 private:
 	QTimer *pingTimer;
 	bool isInRecipientTable(const QString &idDs) const;
 	void initSearchWindow(void);
 	void addContactsToTable(const QList< QVector<QString> > &contactList);
+	void insertContactToRecipentTable(int selRow);
 
 	Action m_action;
 	QTableWidget *m_recipientTableWidget;
