@@ -29,6 +29,7 @@
 #include <QMimeDatabase>
 
 #include "dlg_send_message.h"
+#include "src/gui/datovka.h"
 #include "src/gui/dlg_change_pwd.h"
 #include "src/gui/dlg_contacts.h"
 #include "src/gui/dlg_ds_search.h"
@@ -45,6 +46,7 @@
 #include "src/worker/message_emitter.h"
 #include "src/worker/pool.h"
 #include "src/worker/task_download_credit_info.h"
+#include "src/worker/task_keep_alive.h"
 #include "src/worker/task_send_message.h"
 #include "ui_dlg_send_message.h"
 
@@ -61,8 +63,9 @@
 DlgSendMessage::DlgSendMessage(
     const QList<Task::AccountDescr> &messageDbSetList,
     Action action, qint64 msgId, const QDateTime &deliveryTime,
-    const QString &userName, MainWindow *mv, QWidget *parent)
+    const QString &userName, class MainWindow *mv, QWidget *parent)
     : QDialog(parent),
+    m_keepAliveTimer(),
     m_messageDbSetList(messageDbSetList),
     m_msgID(msgId),
     m_deliveryTime(deliveryTime),
@@ -205,11 +208,9 @@ void DlgSendMessage::initNewMessageDialog(void)
 	    SLOT(collectSendMessageStatus(QString, QString, int, QString,
 	        QString, QString, bool, qint64)));
 
-	pingTimer = new QTimer(this);
-	pingTimer->start(DLG_ISDS_KEEPALIVE_MS);
-
-	connect(pingTimer, SIGNAL(timeout()), this,
+	connect(&m_keepAliveTimer, SIGNAL(timeout()), this,
 	    SLOT(pingIsdsServer()));
+	m_keepAliveTimer.start(DLG_ISDS_KEEPALIVE_MS);
 
 	this->attachmentSizeInfo->setText(
 	    tr("Total size of attachments is %1 B").arg(0));
@@ -268,11 +269,22 @@ void DlgSendMessage::setAccountInfo(int item)
 
 	m_isLogged = true;
 
-	if (!isdsSessions.isConnectedToIsds(m_userName)) {
-		if (!m_mv->connectToIsds(m_userName, m_mv)) {
-			m_isLogged = false;
-		}
+	m_keepAliveTimer.stop();
+	{
+		TaskKeepAlive *task;
+		task = new (std::nothrow) TaskKeepAlive(m_userName);
+		task->setAutoDelete(false);
+		globWorkPool.runSingle(task);
+
+		m_isLogged = task->m_isAlive;
+
+		delete task;
 	}
+	if (!m_isLogged) {
+		m_isLogged = MainWindow::connectToIsds(m_userName, m_mv);
+	}
+	m_keepAliveTimer.start(DLG_ISDS_KEEPALIVE_MS);
+
 	session = isdsSessions.session(m_userName);
 	if (NULL == session) {
 		logErrorNL("%s", "Missing ISDS session.");
@@ -286,8 +298,7 @@ void DlgSendMessage::setAccountInfo(int item)
 		}
 	}
 
-	const AccountModel::SettingsMap &accountInfo =
-	    AccountModel::globAccounts[m_userName];
+	const AcntSettings &accountInfo(AccountModel::globAccounts[m_userName]);
 	m_dbId = globAccountDbPtr->dbId(m_userName + "___True");
 	Q_ASSERT(!m_dbId.isEmpty());
 	m_senderName =
@@ -364,8 +375,7 @@ void DlgSendMessage::tableItemDoubleClicked(QTableWidgetItem *item)
 {
 	debugSlotCall();
 
-	/* Unused. */
-	(void) item;
+	Q_UNUSED(item);
 
 	openAttachmentFile();
 }
@@ -379,10 +389,9 @@ void DlgSendMessage::attachmentDataChanged(const QModelIndex &topLeft,
     const QModelIndex &bottomRight, const QVector<int> &roles)
 /* ========================================================================= */
 {
-	/* Unused. */
-	(void) topLeft;
-	(void) bottomRight;
-	(void) roles;
+	Q_UNUSED(topLeft);
+	Q_UNUSED(bottomRight);
+	Q_UNUSED(roles);
 
 	debugSlotCall();
 
@@ -400,9 +409,8 @@ void DlgSendMessage::attachmentSelectionChanged(const QItemSelection &selected,
 {
 	debugSlotCall();
 
-	/* Unused. */
-	(void) selected;
-	(void) deselected;
+	Q_UNUSED(selected);
+	Q_UNUSED(deselected);
 
 	QModelIndexList selectedIndexes;
 	{
@@ -648,11 +656,11 @@ void DlgSendMessage::fillDlgFromTmpMsg(void)
 void DlgSendMessage::pingIsdsServer(void)
 /* ========================================================================= */
 {
-	if (isdsSessions.isConnectedToIsds(m_userName)) {
-		logInfo("%s\n", "Connection to ISDS is alive :)");
-	} else {
-		logWarning("%s\n", "Connection to ISDS is dead :(");
-	}
+	TaskKeepAlive *task;
+
+	task = new (std::nothrow) TaskKeepAlive(m_userName);
+	task->setAutoDelete(true);
+	globWorkPool.assignHi(task);
 }
 
 
@@ -933,7 +941,7 @@ void DlgSendMessage::openAttachmentFile(void)
 
 	fileName = writeTemporaryFile(fileName, data);
 	if (!fileName.isEmpty()) {
-		QDesktopServices::openUrl(QUrl(URL_FILE_PREFIX + fileName));
+		QDesktopServices::openUrl(QUrl::fromLocalFile(fileName));
 		/* TODO -- Handle openUrl() return value. */
 	} else {
 		QMessageBox::warning(this,
@@ -1191,8 +1199,7 @@ void DlgSendMessage::collectSendMessageStatus(const QString &userName,
 {
 	debugSlotCall();
 
-	/* Unused. */
-	(void) userName;
+	Q_UNUSED(userName);
 
 	if (m_transactIds.end() == m_transactIds.find(transactId)) {
 		/* Nothing found. */
