@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 CZ.NIC
+ * Copyright (C) 2014-2016 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,193 +21,19 @@
  * the two.
  */
 
-
-#include <QDebug>
 #include <QMap>
 #include <QObject>
 #include <QPair>
-#include <QSqlDatabase>
-#include <QSqlError>
-#include <QSqlQuery>
 #include <QString>
-#include <QVariant>
 #include <QVector>
 
-#include "db_tables.h"
-
-
-/* ========================================================================= */
-/*
- * Converts db types strings.
- */
-const QString & entryTypeStr(EntryType entryType)
-/* ========================================================================= */
-{
-	static const QString integer("INTEGER");
-	static const QString text("TEXT");
-	static const QString boolean("BOOLEAN");
-	static const QString datetime("DATETIME");
-	static const QString date("DATE");
-	static const QString invalid;
-
-	switch (entryType) {
-	case DB_INTEGER:
-	case DB_INT_PROCESSING_STATE:
-		return integer;
-		break;
-	case DB_TEXT:
-		return text;
-		break;
-	case DB_BOOLEAN:
-	case DB_BOOL_READ_LOCALLY:
-	case DB_BOOL_ATTACHMENT_DOWNLOADED:
-		return boolean;
-		break;
-	case DB_DATETIME:
-		return datetime;
-		break;
-	case DB_DATE:
-		return date;
-		break;
-	default:
-		Q_ASSERT(0);
-		return invalid;
-		break;
-	}
-}
-
-
-/* ========================================================================= */
-/*
- * Constructor.
- */
-Tbl::Tbl(const QString &name,
-    const QVector< QPair<QString, EntryType> > &attrs,
-    const QMap<QString, AttrProp> &props,
-    const QMap<QString, QString> &colCons,
-    const QString &tblCons)
-    : tabName(name),
-    knownAttrs(attrs),
-    attrPropsSources(props),
-    attrProps(props),
-    colConstraints(colCons),
-    tblConstraint(tblCons)
-/* ========================================================================= */
-{
-}
-
-
-/* ========================================================================= */
-/*
- * Return true if table in database exists.
- */
-bool Tbl::existsInDb(const QSqlDatabase &db) const
-/* ========================================================================= */
-{
-	Q_ASSERT(db.isValid());
-	if (!db.isValid()) {
-		return false;
-	}
-
-	Q_ASSERT(db.isOpen());
-	if (!db.isOpen()) {
-		return false;
-	}
-
-	QSqlQuery query(db);
-	QString queryStr = "SELECT "
-	    "name"
-	    " FROM sqlite_master WHERE "
-	    "type = 'table'"
-	    " and "
-	    "name = :tabName";
-	//qDebug() << queryStr;
-	if (!query.prepare(queryStr)) {
-		return false;
-	}
-	query.bindValue(":tabName", tabName);
-	if (query.exec() && query.isActive() &&
-	    query.first() && query.isValid()) {
-		return true;
-	}
-
-	return false;
-}
-
-
-/* ========================================================================= */
-/*
- * Create empty table in supplied database.
- */
-bool Tbl::createEmpty(QSqlDatabase &db) const
-/* ========================================================================= */
-{
-	Q_ASSERT(db.isValid());
-	if (!db.isValid()) {
-		return false;
-	}
-
-	Q_ASSERT(db.isOpen());
-	if (!db.isOpen()) {
-		return false;
-	}
-
-	QSqlQuery query(db);
-	QString queryStr = "CREATE TABLE IF NOT EXISTS " + tabName + " (\n";
-	for (int i = 0; i < knownAttrs.size(); ++i) {
-		queryStr += "        " + knownAttrs[i].first + " " +
-		    entryTypeStr(knownAttrs[i].second);
-		if (colConstraints.end() !=
-		    colConstraints.find(knownAttrs[i].first)) {
-			queryStr += " " +
-			    colConstraints.value(knownAttrs[i].first);
-		}
-		if ((knownAttrs.size() - 1) != i) {
-			queryStr += ",\n";
-		}
-	}
-	queryStr += tblConstraint;
-	queryStr += "\n)";
-	//qDebug() << queryStr;
-	if (!query.prepare(queryStr) || !query.exec()) {
-		qWarning() << query.lastError();
-		return false;
-	}
-
-	return true;
-}
-
-
-/* ========================================================================= */
-/*
- * Reloads the translation according to the selected
- *     localisation.
- */
-void Tbl::reloadLocalisedDescription(void)
-/* ========================================================================= */
-{
-	attrProps.clear();
-
-	QMap<QString, AttrProp>::const_iterator it = attrPropsSources.begin();
-
-	class AttrProp prop;
-	for (; it != attrPropsSources.end(); ++it) {
-		prop.type = it.value().type;
-		prop.desc = QObject::tr(it.value().desc.toUtf8().constData());
-		attrProps.insert(it.key(), prop);
-	}
-}
-
-
-const QMap<QString, QString> Tbl::emptyColConstraints;
-
-const QString Tbl::emptyTblConstraint;
+#include "src/io/db_tables.h"
 
 
 namespace AccntinfTbl {
 	const QString tabName("account_info");
 
-	const QVector< QPair<QString, EntryType> > knownAttrs = {
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
 	{"key", DB_TEXT}, /* NOT NULL */
 	{"dbID", DB_TEXT}, /* NOT NULL */
 	{"dbType", DB_TEXT},
@@ -247,7 +73,7 @@ namespace AccntinfTbl {
 	    "        CHECK (dbOpenAddressing IN (0, 1))"
 	};
 
-	QMap<QString, AttrProp> attrProps = {
+	QMap<QString, SQLiteTbl::AttrProp> attrProps = {
 	{"key",                    {DB_TEXT, ""}},
 	{"dbID",                   {DB_TEXT, QObject::tr("Data box ID")}},
 	{"dbType",                 {DB_TEXT, QObject::tr("Data box type")}},
@@ -275,7 +101,7 @@ namespace AccntinfTbl {
 	{"dbOpenAddressing",       {DB_BOOLEAN, QObject::tr("Open addressing")}}
 	};
 } /* namespace AccntinfTbl */
-Tbl accntinfTbl(AccntinfTbl::tabName, AccntinfTbl::knownAttrs,
+SQLiteTbl accntinfTbl(AccntinfTbl::tabName, AccntinfTbl::knownAttrs,
     AccntinfTbl::attrProps, AccntinfTbl::colConstraints,
     AccntinfTbl::tblConstraint);
 
@@ -283,7 +109,7 @@ Tbl accntinfTbl(AccntinfTbl::tabName, AccntinfTbl::knownAttrs,
 namespace UserinfTbl {
 	const QString tabName("user_info");
 
-	const QVector< QPair<QString, EntryType> > knownAttrs = {
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
 	{"key", DB_TEXT}, /* NOT NULL */
 	{"userType", DB_TEXT},
 	{"userPrivils", DB_INTEGER},
@@ -315,7 +141,7 @@ namespace UserinfTbl {
 	    "        PRIMARY KEY (key)"
 	};
 
-	QMap<QString, AttrProp> attrProps = {
+	QMap<QString, SQLiteTbl::AttrProp> attrProps = {
 	{"key",                    {DB_TEXT, ""}},
 	{"userType",               {DB_TEXT, QObject::tr("User type")}},
 	{"userPrivils",            {DB_INTEGER, QObject::tr("Permissions")}},
@@ -338,7 +164,7 @@ namespace UserinfTbl {
 	{"caState",                {DB_TEXT, QObject::tr("State of residence")}}
 	};
 } /* namespace UserinfTbl */
-Tbl userinfTbl(UserinfTbl::tabName, UserinfTbl::knownAttrs,
+SQLiteTbl userinfTbl(UserinfTbl::tabName, UserinfTbl::knownAttrs,
     UserinfTbl::attrProps, UserinfTbl::colConstraints,
     UserinfTbl::tblConstraint);
 
@@ -346,7 +172,7 @@ Tbl userinfTbl(UserinfTbl::tabName, UserinfTbl::knownAttrs,
 namespace PwdexpdtTbl {
 	const QString tabName("password_expiration_date");
 
-	const QVector< QPair<QString, EntryType> > knownAttrs = {
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
 	{"key", DB_TEXT}, /* NOT NULL */
 	{"expDate", DB_TEXT}
 	};
@@ -360,12 +186,12 @@ namespace PwdexpdtTbl {
 	    "        PRIMARY KEY (key)"
 	};
 
-	const QMap<QString, AttrProp> attrProps = {
+	const QMap<QString, SQLiteTbl::AttrProp> attrProps = {
 	{"key",     {DB_TEXT, ""}},
 	{"expDate", {DB_TEXT, ""}}
 	};
 } /* namespace PwdexpdtTbl */
-Tbl pwdexpdtTbl(PwdexpdtTbl::tabName, PwdexpdtTbl::knownAttrs,
+SQLiteTbl pwdexpdtTbl(PwdexpdtTbl::tabName, PwdexpdtTbl::knownAttrs,
     PwdexpdtTbl::attrProps, PwdexpdtTbl::colConstraints,
     PwdexpdtTbl::tblConstraint);
 
@@ -373,7 +199,7 @@ Tbl pwdexpdtTbl(PwdexpdtTbl::tabName, PwdexpdtTbl::knownAttrs,
 namespace MsgsTbl {
 	const QString tabName("messages");
 
-	const QVector< QPair<QString, EntryType> > knownAttrs = {
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
 	{"dmID", DB_INTEGER}, /* NOT NULL */
 	{"is_verified", DB_BOOLEAN},
 	{"_origin", DB_TEXT},
@@ -428,7 +254,7 @@ namespace MsgsTbl {
 	    "        CHECK (dmAllowSubstDelivery IN (0, 1))"
 	);
 
-	const QMap<QString, AttrProp> attrProps = {
+	const QMap<QString, SQLiteTbl::AttrProp> attrProps = {
 	{"dmID",                  {DB_INTEGER, QObject::tr("ID")}},
 	{"is_verified",           {DB_BOOLEAN, ""}},
 	{"_origin",               {DB_TEXT, ""}},
@@ -455,8 +281,8 @@ namespace MsgsTbl {
 	{"dmLegalTitleSect",      {DB_TEXT, QObject::tr("Selection")}},
 	{"dmLegalTitlePar",       {DB_TEXT, QObject::tr("Paragraph")}},
 	{"dmLegalTitlePoint",     {DB_TEXT, QObject::tr("Letter")}},
-	{"dmPersonalDelivery",    {DB_BOOLEAN, ""}},
-	{"dmAllowSubstDelivery",  {DB_BOOLEAN, ""}},
+	{"dmPersonalDelivery",    {DB_BOOLEAN, QObject::tr("Personal delivery")}},
+	{"dmAllowSubstDelivery",  {DB_BOOLEAN, QObject::tr("Enable delivery by fiction")}},
 	{"dmQTimestamp",          {DB_TEXT, ""}},
 	{"dmDeliveryTime",        {DB_DATETIME, QObject::tr("Delivered")}},
 	{"dmAcceptanceTime",      {DB_DATETIME, QObject::tr("Accepted")}},
@@ -465,14 +291,14 @@ namespace MsgsTbl {
 	{"_dmType",               {DB_TEXT, ""}}
 	};
 } /* namespace MsgsTbl */
-Tbl msgsTbl(MsgsTbl::tabName, MsgsTbl::knownAttrs, MsgsTbl::attrProps,
+SQLiteTbl msgsTbl(MsgsTbl::tabName, MsgsTbl::knownAttrs, MsgsTbl::attrProps,
     MsgsTbl::colConstraints, MsgsTbl::tblConstraint);
 
 
 namespace FlsTbl {
 	const QString tabName("files");
 
-	const QVector< QPair<QString, EntryType> > knownAttrs = {
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
 	{"id", DB_INTEGER}, /* NOT NULL*/
 	{"message_id",  DB_INTEGER},
 	{"_dmFileDescr", DB_TEXT},
@@ -498,7 +324,7 @@ namespace FlsTbl {
 	    "        FOREIGN KEY(message_id) REFERENCES messages (dmID)"
 	);
 
-	const QMap<QString, AttrProp> attrProps = {
+	const QMap<QString, SQLiteTbl::AttrProp> attrProps = {
 	{"id",               {DB_INTEGER, ""}},
 	{"message_id",       {DB_INTEGER, ""}},
 	{"_dmFileDescr",     {DB_TEXT, QObject::tr("File name")}},
@@ -510,14 +336,14 @@ namespace FlsTbl {
 	{"dmEncodedContent", {DB_TEXT, ""}}
 	};
 } /* namespace FlsTbl */
-Tbl flsTbl(FlsTbl::tabName, FlsTbl::knownAttrs, FlsTbl::attrProps,
+SQLiteTbl flsTbl(FlsTbl::tabName, FlsTbl::knownAttrs, FlsTbl::attrProps,
     FlsTbl::colConstraints, FlsTbl::tblConstraint);
 
 
 namespace HshsTbl {
 	const QString tabName("hashes");
 
-	const QVector< QPair<QString, EntryType> > knownAttrs = {
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
 	{"id", DB_INTEGER}, /* NOT NULL */
 	{"message_id", DB_INTEGER},
 	{"value", DB_TEXT},
@@ -538,21 +364,21 @@ namespace HshsTbl {
 	    "        FOREIGN KEY(message_id) REFERENCES messages (dmID)"
 	);
 
-	const QMap<QString, AttrProp> attrProps = {
+	const QMap<QString, SQLiteTbl::AttrProp> attrProps = {
 	{"id",         {DB_INTEGER, ""}},
 	{"message_id", {DB_INTEGER, ""}},
 	{"value",      {DB_TEXT, ""}},
 	{"_algorithm", {DB_TEXT, ""}}
 	};
 } /* namespace HshsTbl */
-Tbl hshsTbl(HshsTbl::tabName, HshsTbl::knownAttrs, HshsTbl::attrProps,
+SQLiteTbl hshsTbl(HshsTbl::tabName, HshsTbl::knownAttrs, HshsTbl::attrProps,
     HshsTbl::colConstraints, HshsTbl::tblConstraint);
 
 
 namespace EvntsTbl {
 	const QString tabName("events");
 
-	const QVector< QPair<QString, EntryType> > knownAttrs = {
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
 	{"id", DB_INTEGER}, /* NOT NULL */
 	{"message_id", DB_INTEGER},
 	{"dmEventTime", DB_TEXT},
@@ -573,21 +399,21 @@ namespace EvntsTbl {
 	    "        FOREIGN KEY(message_id) REFERENCES messages (dmID)"
 	);
 
-	const QMap<QString, AttrProp> attrProps = {
+	const QMap<QString, SQLiteTbl::AttrProp> attrProps = {
 	{"id",           {DB_INTEGER, ""}},
 	{"message_id",   {DB_INTEGER, ""}},
 	{"dmEventTime",  {DB_TEXT, ""}},
 	{"dmEventDescr", {DB_TEXT, ""}}
 	};
 } /* namespace EvntsTbl */
-Tbl evntsTbl(EvntsTbl::tabName, EvntsTbl::knownAttrs,
+SQLiteTbl evntsTbl(EvntsTbl::tabName, EvntsTbl::knownAttrs,
     EvntsTbl::attrProps, EvntsTbl::colConstraints, EvntsTbl::tblConstraint);
 
 
 namespace RwmsgdtTbl {
 	const QString tabName("raw_message_data");
 
-	const QVector< QPair<QString, EntryType> > knownAttrs = {
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
 	{"message_id", DB_INTEGER}, /* NOT NULL */
 	{"message_type", DB_INTEGER},
 	{"data", DB_TEXT}
@@ -607,13 +433,13 @@ namespace RwmsgdtTbl {
 	    "        FOREIGN KEY(message_id) REFERENCES messages (dmID)"
 	);
 
-	const QMap<QString, AttrProp> attrProps = {
+	const QMap<QString, SQLiteTbl::AttrProp> attrProps = {
 	{"message_id",   {DB_INTEGER, ""}},
 	{"message_type", {DB_INTEGER, ""}},
 	{"data",         {DB_TEXT, ""}}
 	};
 } /* namespace RwmsgdtTbl */
-Tbl rwmsgdtTbl(RwmsgdtTbl::tabName, RwmsgdtTbl::knownAttrs,
+SQLiteTbl rwmsgdtTbl(RwmsgdtTbl::tabName, RwmsgdtTbl::knownAttrs,
     RwmsgdtTbl::attrProps, RwmsgdtTbl::colConstraints,
     RwmsgdtTbl::tblConstraint);
 
@@ -621,7 +447,7 @@ Tbl rwmsgdtTbl(RwmsgdtTbl::tabName, RwmsgdtTbl::knownAttrs,
 namespace RwdlvrinfdtTbl {
 	const QString tabName("raw_delivery_info_data");
 
-	const QVector< QPair<QString, EntryType> > knownAttrs = {
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
 	{"message_id", DB_INTEGER}, /* NOT NULL */
 	{"data", DB_TEXT}
 	/*
@@ -640,12 +466,12 @@ namespace RwdlvrinfdtTbl {
 	    "        FOREIGN KEY(message_id) REFERENCES messages (dmID)"
 	);
 
-	const QMap<QString, AttrProp> attrProps = {
+	const QMap<QString, SQLiteTbl::AttrProp> attrProps = {
 	{"message_id", {DB_INTEGER, ""}},
 	{"data",       {DB_TEXT, ""}}
 	};
 } /* namespace RwdlvrinfdtTbl */
-Tbl rwdlvrinfdtTbl(RwdlvrinfdtTbl::tabName, RwdlvrinfdtTbl::knownAttrs,
+SQLiteTbl rwdlvrinfdtTbl(RwdlvrinfdtTbl::tabName, RwdlvrinfdtTbl::knownAttrs,
     RwdlvrinfdtTbl::attrProps, RwdlvrinfdtTbl::colConstraints,
     RwdlvrinfdtTbl::tblConstraint);
 
@@ -653,7 +479,7 @@ Tbl rwdlvrinfdtTbl(RwdlvrinfdtTbl::tabName, RwdlvrinfdtTbl::knownAttrs,
 namespace SmsgdtTbl {
 	const QString tabName("supplementary_message_data");
 
-	const QVector< QPair<QString, EntryType> > knownAttrs = {
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
 	{"message_id", DB_INTEGER}, /* NOT NULL */
 	{"message_type", DB_INTEGER},
 	{"read_locally", DB_BOOL_READ_LOCALLY},
@@ -677,7 +503,7 @@ namespace SmsgdtTbl {
 	    "        CHECK (read_locally IN (0, 1))"
 	);
 
-	const QMap<QString, AttrProp> attrProps = {
+	const QMap<QString, SQLiteTbl::AttrProp> attrProps = {
 	{"message_id",    {DB_INTEGER, ""}},
 	{"message_type",  {DB_INTEGER, ""}},
 	{"read_locally",  {DB_BOOL_READ_LOCALLY, QObject::tr("Read locally")}},
@@ -685,14 +511,14 @@ namespace SmsgdtTbl {
 	{"custom_data",   {DB_TEXT, ""}}
 	};
 } /* namespace SmsgdtTbl */
-Tbl smsgdtTbl(SmsgdtTbl::tabName, SmsgdtTbl::knownAttrs,
+SQLiteTbl smsgdtTbl(SmsgdtTbl::tabName, SmsgdtTbl::knownAttrs,
     SmsgdtTbl::attrProps, SmsgdtTbl::colConstraints, SmsgdtTbl::tblConstraint);
 
 
 namespace CrtdtTbl {
 	const QString tabName("certificate_data");
 
-	const QVector< QPair<QString, EntryType> > knownAttrs = {
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
 	{"id", DB_INTEGER}, /* NOT NULL */
 	{"der_data", DB_TEXT}
 	/*
@@ -711,19 +537,19 @@ namespace CrtdtTbl {
 	    "        UNIQUE (der_data)"
 	);
 
-	const QMap<QString, AttrProp> attrProps = {
+	const QMap<QString, SQLiteTbl::AttrProp> attrProps = {
 	{"id",       {DB_INTEGER, ""}},
 	{"der_data", {DB_TEXT, ""}}
 	};
 } /* namespace CrtdtTbl */
-Tbl crtdtTbl(CrtdtTbl::tabName, CrtdtTbl::knownAttrs,
+SQLiteTbl crtdtTbl(CrtdtTbl::tabName, CrtdtTbl::knownAttrs,
     CrtdtTbl::attrProps, CrtdtTbl::colConstraints, CrtdtTbl::tblConstraint);
 
 
 namespace MsgcrtdtTbl {
 	const QString tabName("message_certificate_data");
 
-	const QVector< QPair<QString, EntryType> > knownAttrs = {
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
 	{"message_id", DB_INTEGER},
 	{"certificate_id", DB_INTEGER}
 	/*
@@ -740,12 +566,12 @@ namespace MsgcrtdtTbl {
 	    "        FOREIGN KEY(certificate_id) REFERENCES certificate_data (id)"
 	);
 
-	const QMap<QString, AttrProp> attrProps = {
+	const QMap<QString, SQLiteTbl::AttrProp> attrProps = {
 	{"message_id",     {DB_INTEGER, ""}},
 	{"certificate_id", {DB_INTEGER, ""}}
 	};
 } /* namespace MsgcrtdtTbl */
-Tbl msgcrtdtTbl(MsgcrtdtTbl::tabName, MsgcrtdtTbl::knownAttrs,
+SQLiteTbl msgcrtdtTbl(MsgcrtdtTbl::tabName, MsgcrtdtTbl::knownAttrs,
     MsgcrtdtTbl::attrProps, MsgcrtdtTbl::colConstraints,
     MsgcrtdtTbl::tblConstraint);
 
@@ -753,7 +579,7 @@ Tbl msgcrtdtTbl(MsgcrtdtTbl::tabName, MsgcrtdtTbl::knownAttrs,
 namespace PrcstTbl {
 	const QString tabName("process_state");
 
-	const QVector< QPair<QString, EntryType> > knownAttrs = {
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
 	{"message_id", DB_INTEGER}, /* NOT NULL */
 	{"state",      DB_INTEGER}
 	/*
@@ -772,12 +598,85 @@ namespace PrcstTbl {
 	    "        FOREIGN KEY(message_id) REFERENCES messages (dmID)"
 	);
 
-	const QMap<QString, AttrProp> attrProps = {
+	const QMap<QString, SQLiteTbl::AttrProp> attrProps = {
 	{"message_id", {DB_INTEGER, ""}},
 	{"state",      {DB_INTEGER, ""}}
 	};
 } /* namespace PrcstTbl */
-Tbl prcstTbl(PrcstTbl::tabName, PrcstTbl::knownAttrs,
+SQLiteTbl prcstTbl(PrcstTbl::tabName, PrcstTbl::knownAttrs,
     PrcstTbl::attrProps, PrcstTbl::colConstraints,
     PrcstTbl::tblConstraint);
 
+
+namespace TagTbl {
+	const QString tabName("tag");
+
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
+	{"id", DB_INTEGER}, /* NOT NULL */
+	{"tag_name", DB_TEXT}, /* NOT NULL */
+	{"tag_color", DB_TEXT},
+	/*
+	 * PRIMARY KEY (id),
+	 * UNIQUE (tag_name)
+	 */
+	};
+
+	const QMap<QString, QString> colConstraints = {
+	    {"id", "NOT NULL"},
+	    {"tag_name", "NOT NULL"}
+	};
+
+	const QString &tblConstraint(
+	    ",\n"
+	    "        PRIMARY KEY (id),\n"
+	    "        UNIQUE (tag_name)"
+	);
+
+	const QMap<QString, SQLiteTbl::AttrProp> attrProps = {
+	{"id",       {DB_INTEGER, ""}},
+	{"tag_name", {DB_TEXT, ""}},
+	{"tag_color", {DB_TEXT, ""}}
+	};
+} /* namespace TagTbl */
+SQLiteTbl tagTbl(TagTbl::tabName, TagTbl::knownAttrs,
+    TagTbl::attrProps, TagTbl::colConstraints,
+    TagTbl::tblConstraint);
+
+
+namespace MsgTagsTbl {
+	const QString tabName("message_tags");
+
+	const QVector< QPair<QString, enum EntryType> > knownAttrs = {
+	{"id", DB_INTEGER}, /* NOT NULL */
+	{"user_name", DB_TEXT}, /* NOT NULL */
+	{"message_id", DB_INTEGER}, /* NOT NULL */
+	{"tag_id", DB_INTEGER} /* NOT NULL */
+	/*
+	 * PRIMARY KEY (id),
+	 * FOREIGN KEY(tag_id) REFERENCES tag ("id")
+	 */
+	};
+
+	const QMap<QString, QString> colConstraints = {
+	    {"id", "NOT NULL"},
+	    {"user_name", "NOT NULL"},
+	    {"message_id", "NOT NULL"},
+	    {"tag_id", "NOT NULL"}
+	};
+
+	const QString &tblConstraint(
+	    ",\n"
+	    "        PRIMARY KEY (id),\n"
+	    "        FOREIGN KEY(tag_id) REFERENCES tag (id)"
+	);
+
+	const QMap<QString, SQLiteTbl::AttrProp> attrProps = {
+	{"id",           {DB_INTEGER, ""}},
+	{"user_name",     {DB_TEXT, ""}},
+	{"message_id",   {DB_INTEGER, ""}},
+	{"tag_id",       {DB_INTEGER, ""}}
+	};
+} /* namespace MsgTagsTbl */
+SQLiteTbl msgtagsTbl(MsgTagsTbl::tabName, MsgTagsTbl::knownAttrs,
+    MsgTagsTbl::attrProps, MsgTagsTbl::colConstraints,
+    MsgTagsTbl::tblConstraint);
