@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 CZ.NIC
+ * Copyright (C) 2014-2016 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include "isds_sessions.h"
 #include "src/gui/dlg_import_zfo.h"
 #include "src/log/log.h"
+#include "src/models/accounts_model.h"
 #include "src/settings/preferences.h"
 
 
@@ -87,8 +88,8 @@ void logCallback(isds_log_facility facility, isds_log_level level,
     const char *message, int length, void *data)
 /* ========================================================================= */
 {
-	(void) data;
-	(void) length;
+	Q_UNUSED(data);
+	Q_UNUSED(length);
 	const char *logFac = logFacilityName(facility);
 
 	switch (level) {
@@ -131,7 +132,16 @@ GlobIsdsSessions::GlobIsdsSessions(void)
 	isds_set_log_callback(logCallback, NULL);
 
 	/* Logging. */
+#if !defined(Q_OS_WIN)
 	isds_set_logging(ILF_ALL, ILL_ALL);
+#else /* defined(Q_OS_WIN) */
+	/*
+	 * There is a issue related to logging when using libisds compiled
+	 * with MinGW. See https://gitlab.labs.nic.cz/labs/qdatovka/issues/233
+	 * for more details.
+	 */
+	isds_set_logging(ILF_ALL, ILL_INFO);
+#endif /* !defined(Q_OS_WIN) */
 }
 
 
@@ -927,14 +937,8 @@ fail:
 	return NULL;
 }
 
-
-/* ========================================================================= */
-/*
- * Create a isds message from zfo file.
- */
-struct isds_message * loadZfoFile(struct isds_ctx *isdsSession,
-    const QString &fileName, int zfoType)
-/* ========================================================================= */
+struct isds_message *loadZfoData(struct isds_ctx *isdsSession,
+    const QByteArray &rawMsgData, int zfoType)
 {
 	isds_error status;
 	isds_raw_type raw_type;
@@ -942,35 +946,25 @@ struct isds_message * loadZfoFile(struct isds_ctx *isdsSession,
 
 	Q_ASSERT(NULL != isdsSession);
 
-	QFile file(fileName);
-	QByteArray content;
-
-	if (!file.open(QIODevice::ReadOnly)) {
-		qWarning() << "Cannot open file" << fileName;
-		goto fail;
-	}
-
-	content = file.readAll();
-	file.close();
-
-	status = isds_guess_raw_type(isdsSession, &raw_type, content.data(),
-	    content.size());
+	status = isds_guess_raw_type(isdsSession, &raw_type, rawMsgData.data(),
+	    rawMsgData.size());
 	if (IE_SUCCESS != status) {
-		qWarning() << "Cannot guess content type of file" << fileName;
+		logErrorNL("%s", "Cannot guess message content type.");
 		goto fail;
 	}
 
 	if (zfoType == ImportZFODialog::IMPORT_MESSAGE_ZFO) {
 		status = isds_load_message(isdsSession, raw_type,
-		    content.data(), content.size(), &message, BUFFER_COPY);
+		    rawMsgData.data(), rawMsgData.size(), &message,
+		    BUFFER_COPY);
 	} else {
 		status = isds_load_delivery_info(isdsSession, raw_type,
-		    content.data(), content.size(), &message, BUFFER_COPY);
+		    rawMsgData.data(), rawMsgData.size(), &message,
+		    BUFFER_COPY);
 	}
 
 	if (IE_SUCCESS != status) {
-		qWarning() << "Error while loading data from file"
-		    << fileName;
+		logErrorNL("%s", "Cannot load message content.");
 		goto fail;
 	}
 
@@ -982,3 +976,28 @@ fail:
 	}
 	return NULL;
 }
+
+struct isds_message *loadZfoFile(struct isds_ctx *isdsSession,
+    const QString &fileName, int zfoType)
+{
+	QFile file(fileName);
+
+	if (!file.open(QIODevice::ReadOnly)) {
+		logErrorNL("Cannot open file '%s'.",
+		    fileName.toUtf8().constData());
+		return NULL;
+	}
+
+	QByteArray content(file.readAll());
+	file.close();
+
+	struct isds_message *message =
+	    loadZfoData(isdsSession, content, zfoType);
+	if (NULL == message) {
+		logErrorNL("Error while loading data from file '%s'.",
+		    fileName.toUtf8().constData());
+		return NULL;
+	}
+
+	return message;
+}
