@@ -36,13 +36,16 @@
 #include "src/crypto/crypto_threads.h"
 #include "src/crypto/crypto_funcs.h"
 #include "src/gui/datovka.h"
+#include "src/gui/dlg_about.h"
 #include "src/gui/dlg_view_zfo.h"
 #include "src/io/db_tables.h"
 #include "src/io/file_downloader.h"
 #include "src/io/filesystem.h"
-#include "src/io/message_db.h"
 #include "src/io/message_db_set_container.h"
+#include "src/io/tag_db.h"
+#include "src/io/sqlite/db.h"
 #include "src/log/log.h"
+#include "src/models/accounts_model.h"
 #include "src/settings/proxy.h"
 #include "src/single/single_instance.h"
 #include "src/worker/pool.h"
@@ -56,6 +59,9 @@
 #define DEBUG_OPT "debug"
 #define DEBUG_VERBOSITY_OPT "debug-verbosity"
 
+#define RUN_MODE_GUI 0
+#define RUN_MODE_CLI 1
+#define RUN_MODE_ZFO 2
 
 /* ========================================================================= */
 static
@@ -415,7 +421,8 @@ int main(int argc, char *argv[])
 
 	/* TODO -- Make the following assignments configurable. */
 	QCoreApplication::setApplicationName("Datovka");
-	QCoreApplication::setApplicationVersion(VERSION);
+	QCoreApplication::setApplicationVersion(VERSION " -- [" +
+	    DlgAbout::libraryDependencies().join("; ") + "]");
 
 	qInstallMessageHandler(globalLogOutput);
 
@@ -437,6 +444,30 @@ int main(int argc, char *argv[])
 	}
 
 	QStringList cmdLineFileNames = parser.positionalArguments();
+	QStringList inOptions = parser.optionNames();
+	QStringList serList;
+
+	// check if any CLI service was set from command line
+	for (int i = 0; i < inOptions.count(); ++i) {
+		for (int j = 0; j < serviceList.count(); ++j) {
+			if (inOptions.at(i) == serviceList.at(j)) {
+				serList.append(inOptions.at(i));
+				break;
+			}
+		}
+	}
+
+	short runMode = RUN_MODE_GUI;
+	QSplashScreen * splash = new QSplashScreen;
+
+	if (!serList.isEmpty()) {
+		runMode = RUN_MODE_CLI;
+	} else if (!cmdLineFileNames.isEmpty()) {
+		runMode = RUN_MODE_ZFO;
+	} else {
+		splash->setPixmap(QPixmap(":/splash/datovka-splash.png"));
+		splash->show();
+	}
 
 	/* Ensure only one instance with given configuration file. */
 	SingleInstance singleInstance(globPref.loadConfPath());
@@ -526,9 +557,9 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	if (!MessageDbSet::dbDriverSupport()) {
+	if (!SQLiteDb::dbDriverSupport()) {
 		logError("Cannot load database driver '%s'.\n",
-		    MessageDbSet::dbDriverType.toUtf8().constData());
+		    SQLiteDb::dbDriverType.toUtf8().constData());
 		/* TODO -- throw a dialog notifying the user. */
 		return EXIT_FAILURE;
 	}
@@ -540,6 +571,14 @@ int main(int argc, char *argv[])
 
 	loadLocalisation(app);
 
+	/* set splash action text */
+	if (runMode == RUN_MODE_GUI) {
+		Qt::Alignment align = Qt::AlignCenter;
+		splash->showMessage(QObject::tr("Application is loading..."),
+		align, Qt::white);
+		app.processEvents();
+	}
+
 	/* Localise description in tables. */
 	accntinfTbl.reloadLocalisedDescription();
 	userinfTbl.reloadLocalisedDescription();
@@ -562,20 +601,41 @@ int main(int argc, char *argv[])
 	 *
 	 * TODO -- Solve the problem of this globally accessible structures.
 	 */
-	AccountDb globAccountDb("accountDb");
-	globAccountDbPtr = &globAccountDb;
-	DbContainer globMessageDbs("GLOBALDBS");
-	globMessageDbsPtr = &globMessageDbs;
-
 	{
+		globAccountDbPtr = new (std::nothrow) AccountDb("accountDb");
+		if (0 == globAccountDbPtr) {
+			logErrorNL("%s", "Cannot allocate account db.");
+			return EXIT_FAILURE;
+		}
 		/* Open accounts database. */
 		if (!globAccountDbPtr->openDb(globPref.accountDbPath())) {
 			logErrorNL("Error opening account db '%s'.",
 			    globPref.accountDbPath().toUtf8().constData());
 			return EXIT_FAILURE;
 		}
+
+		/* Create message DB container. */
+		globMessageDbsPtr = new (std::nothrow) DbContainer("GLOBALDBS");
+		if (0 == globMessageDbsPtr) {
+			logErrorNL("%s",
+			    "Cannot allocate message db container.");
+			return EXIT_FAILURE;
+		}
+
+		globTagDbPtr = new (std::nothrow) TagDb("tagDb");
+		if (0 == globTagDbPtr) {
+			logErrorNL("%s", "Cannot allocate tag db.");
+			return EXIT_FAILURE;
+		}
+		/* Open tags database. */
+		if (!globTagDbPtr->openDb(globPref.tagDbPath())) {
+			logErrorNL("Error opening tag db '%s'.",
+			    globPref.tagDbPath().toUtf8().constData());
+			return EXIT_FAILURE;
+		}
 	}
 
+
 	int ret = EXIT_SUCCESS;
 
 	/* Parse account information. */
@@ -591,33 +651,23 @@ int main(int argc, char *argv[])
 	globWorkPool.start();
 	logInfo("%s\n", "Worker pool started.");
 
-	QStringList inOptions = parser.optionNames();
-	QStringList serList;
-
-	// check if any CLI service was set from command line
-	for (int i = 0; i < inOptions.count(); ++i) {
-		for (int j = 0; j < serviceList.count(); ++j) {
-			if (inOptions.at(i) == serviceList.at(j)) {
-				serList.append(inOptions.at(i));
-				break;
-			}
-		}
-	}
-
-	// if any CLI service was set run CLI
-	if (!serList.isEmpty()) {
+	if (runMode == RUN_MODE_CLI) {
+		delete splash;
 		ret = doCLI(serList, parser);
-	} else if (cmdLineFileNames.isEmpty()) {
-		MainWindow mainwin;
-		mainwin.show();
-		ret = (0 == app.exec()) ? EXIT_SUCCESS : EXIT_FAILURE;
-	} else {
+	} else if (runMode == RUN_MODE_ZFO) {
+		delete splash;
 		foreach (const QString &fileName, cmdLineFileNames) {
 			ret = showZfo(fileName);
 			if (0 != ret) {
 				break;
 			}
 		}
+	} else {
+		MainWindow mainwin;
+		mainwin.show();
+		splash->finish(&mainwin);
+		delete splash;
+		ret = (0 == app.exec()) ? EXIT_SUCCESS : EXIT_FAILURE;
 	}
 
 	/* Wait until all threads finished. */
@@ -639,5 +689,18 @@ int main(int argc, char *argv[])
 	 */
 	//crypto_cleanup_threads();
 
+	if (0 != globTagDbPtr) {
+		delete globTagDbPtr;
+		globTagDbPtr = 0;
+	}
+	if (0 != globMessageDbsPtr) {
+		delete globMessageDbsPtr;
+		globMessageDbsPtr = 0;
+	}
+	if (0 != globAccountDbPtr) {
+		delete globAccountDbPtr;
+		globAccountDbPtr = 0;
+	}
+
 	return ret;
 }
