@@ -21,6 +21,7 @@
  * the two.
  */
 
+#include <cinttypes>
 #include <QThread>
 
 #include "src/io/dbs.h"
@@ -32,14 +33,15 @@
 
 TaskDownloadMessage::TaskDownloadMessage(const QString &userName,
     MessageDbSet *dbSet, enum MessageDirection msgDirect,
-    qint64 dmId, const QDateTime &dTime)
+    qint64 dmId, const QDateTime &dTime, bool listScheduled)
     : m_result(DM_ERR),
     m_isdsError(),
     m_isdsLongError(),
     m_mId(dmId, dTime),
     m_userName(userName),
     m_dbSet(dbSet),
-    m_msgDirect(msgDirect)
+    m_msgDirect(msgDirect),
+    m_listScheduled(listScheduled)
 {
 	Q_ASSERT(0 != dbSet);
 	Q_ASSERT(0 <= dmId);
@@ -73,7 +75,7 @@ void TaskDownloadMessage::run(void)
 	/* ### Worker task begin. ### */
 
 	logDebugLv1NL("%s", "-----------------------------------------------");
-	logDebugLv1NL("Downloading %s message '%d' for account '%s'.",
+	logDebugLv1NL("Downloading %s message '%" PRId64 "' for account '%s'.",
 	    (MSG_RECEIVED == m_msgDirect) ? "received" : "sent", m_mId.dmId,
 	    AccountModel::globAccounts[m_userName].accountName().toUtf8().constData());
 	logDebugLv1NL("%s", "-----------------------------------------------");
@@ -82,11 +84,12 @@ void TaskDownloadMessage::run(void)
 	    *m_dbSet, m_isdsError, m_isdsLongError, PL_DOWNLOAD_MESSAGE);
 
 	if (DM_SUCCESS == m_result) {
-		logDebugLv1NL("Done downloading message '%d' for account '%s'.",
+		logDebugLv1NL(
+		    "Done downloading message '%" PRId64 "' for account '%s'.",
 		    m_mId.dmId,
 		    AccountModel::globAccounts[m_userName].accountName().toUtf8().constData());
 	} else {
-		logErrorNL("Downloading message '%d' for account '%s' failed.",
+		logErrorNL("Downloading message '%" PRId64 "' for account '%s' failed.",
 		    m_mId.dmId,
 		    AccountModel::globAccounts[m_userName].accountName().toUtf8().constData());
 	}
@@ -94,7 +97,8 @@ void TaskDownloadMessage::run(void)
 	emit globMsgProcEmitter.downloadMessageFinished(m_userName, m_mId.dmId,
 	    m_mId.deliveryTime, m_result,
 	    (!m_isdsError.isEmpty() || !m_isdsLongError.isEmpty()) ?
-	        m_isdsError + " " + m_isdsLongError : "");
+	        m_isdsError + " " + m_isdsLongError : "",
+	    m_listScheduled);
 
 	emit globMsgProcEmitter.progressChange(PL_IDLE, 0);
 
@@ -172,7 +176,8 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadMessage(
 {
 	debugFuncCall();
 
-	logDebugLv0NL("Trying to download complete message '%d'", mId.dmId);
+	logDebugLv0NL("Trying to download complete message '%" PRId64 "'",
+	    mId.dmId);
 
 	emit globMsgProcEmitter.progressChange(progressLabel, 0);
 
@@ -265,10 +270,12 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadMessage(
 	/* Download and save delivery info and message events */
 	if (DM_SUCCESS == downloadDeliveryInfo(userName, mId.dmId, signedMsg,
 	        dbSet, error, longError)) {
-		logDebugLv0NL("Delivery info of message '%d' processed.",
+		logDebugLv0NL(
+		    "Delivery info of message '%" PRId64 "' processed.",
 		    mId.dmId);
 	} else {
-		logErrorNL("Processing delivery info of message '%d' failed.",
+		logErrorNL(
+		    "Processing delivery info of message '%" PRId64 "' failed.",
 		    mId.dmId);
 	}
 
@@ -282,11 +289,12 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadMessage(
 		/*  Mark this message as downloaded in ISDS */
 		if (DM_SUCCESS == markMessageAsDownloaded(userName, mId.dmId,
 		        error, longError)) {
-			logDebugLv0NL("Message '%d' marked as downloaded.",
+			logDebugLv0NL(
+			    "Message '%" PRId64 "' marked as downloaded.",
 			    mId.dmId);
 		} else {
 			logErrorNL(
-			    "Marking message '%d' as downloaded failed.",
+			    "Marking message '%" PRId64 "' as downloaded failed.",
 			    mId.dmId);
 		}
 	}
@@ -333,11 +341,11 @@ enum TaskDownloadMessage::Result TaskDownloadMessage::downloadMessageAuthor(
 	if (messageDb.updateMessageAuthorInfo(dmId,
 	        convertSenderTypeToString((int) *sender_type), sender_name)) {
 		logDebugLv0NL(
-		    "Author information of message '%d' were updated.",
+		    "Author information of message '%" PRId64 "' were updated.",
 		    dmId);
 	} else {
 		logErrorNL(
-		    "Updating author information of message '%d' failed.",
+		    "Updating author information of message '%" PRId64 "' failed.",
 		    dmId);
 	}
 
