@@ -21,6 +21,7 @@
  * the two.
  */
 
+#include <cinttypes>
 #include <QThread>
 
 #include "src/io/dbs.h"
@@ -267,7 +268,7 @@ enum TaskDownloadMessageList::Result TaskDownloadMessageList::downloadMessageLis
 
 				task = new (std::nothrow) TaskDownloadMessage(
 				    userName, &dbSet, msgDirect, dmId,
-				    deliveryTime);
+				    deliveryTime, true);
 				task->setAutoDelete(true);
 				globWorkPool.assignLo(task, WorkerPool::PREPEND);
 			}
@@ -297,7 +298,7 @@ enum TaskDownloadMessageList::Result TaskDownloadMessageList::downloadMessageLis
 
 					task = new (std::nothrow) TaskDownloadMessage(
 					    userName, &dbSet, msgDirect, dmId,
-					    deliveryTime);
+					    deliveryTime, true);
 					task->setAutoDelete(true);
 					globWorkPool.assignLo(task,
 					    WorkerPool::PREPEND);
@@ -319,7 +320,7 @@ enum TaskDownloadMessageList::Result TaskDownloadMessageList::downloadMessageLis
 
 				task = new (std::nothrow) TaskDownloadMessage(
 				    userName, &dbSet, msgDirect, dmId,
-				    deliveryTime);
+				    deliveryTime, true);
 				task->setAutoDelete(true);
 				globWorkPool.assignLo(task, WorkerPool::PREPEND);
 			}
@@ -458,11 +459,11 @@ enum TaskDownloadMessageList::Result TaskDownloadMessageList::updateMessageState
 	        convertHexToDecIndex(*envel->dmMessageStatus) : 0)) {
 		/* Updated message envelope delivery info in db. */
 		logDebugLv0NL(
-		    "Delivery information of message '%d' were updated.",
+		    "Delivery information of message '%" PRId64 "' were updated.",
 		    dmID);
 	} else {
 		logErrorNL(
-		    "Updating delivery information of message '%d' failed.",
+		    "Updating delivery information of message '%" PRId64 "' failed.",
 		    dmID);
 	}
 
