@@ -21,6 +21,7 @@
  * the two.
  */
 
+#include <cinttypes>
 #include <cstdlib>
 #include <cstring>
 #include <QThread>
@@ -29,6 +30,7 @@
 #include "src/io/dbs.h"
 #include "src/io/isds_sessions.h"
 #include "src/log/log.h"
+#include "src/models/accounts_model.h"
 #include "src/worker/message_emitter.h"
 #include "src/worker/task_send_message.h"
 
@@ -498,8 +500,9 @@ enum TaskSendMessage::Result TaskSendMessage::sendMessage(
 		        senderName, message->envelope->dbIDRecipient,
 		        recipientName, recipientAddress,
 		        envelope->dmAnnotation)) {
-			logErrorNL("Cannot insert newly sent message '%d' "
-			    "into database.", dmId);
+			logErrorNL(
+			    "Cannot insert newly sent message '%" PRId64 "' into database.",
+			    dmId);
 			ret = SM_DB_INS_ERR;
 			goto fail;
 		}
